﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Xenon.Text.YokonagaEditor
{

    /// <summary>
    /// 定義：　エディター上で、横幅半分のサイズで表示される文字を、ここでは「半角」と呼ぶことにします。
    /// </summary>
    public abstract class Hankaku
    {

        public static int Length(string src)
        {
            char[] chars = src.ToCharArray();

            int length = 0;
            foreach (char ch in chars)
            {
                if (Hankaku.IsHankaku(ch))
                {
                    length++;
                }
                else
                {
                    length += 2;
                }
            }

            return length;
        }

        /// <summary>
        /// 半角を長さ1、全角を長さ2で計算し、長さcolumn1、長さcolumn2 の地点で文字列を分割することで、
        /// 要素が３つ（[0]～[2]）の文字列を返します。
        /// </summary>
        /// <param name="column1"></param>
        /// <param name="column2"></param>
        /// <returns></returns>
        public static void SplitTri(out string[] result, string src, int column1, int column2)
        {
            StringBuilder sb1 = new StringBuilder();
            StringBuilder sb2 = new StringBuilder();
            StringBuilder sb3 = new StringBuilder();


            char[] chars = src.ToCharArray();

            int c = 0; // 文字インデックス
            int length = 0; // 長さ

            // column1より前
            while (c < chars.Length && length < column1)
            {
                sb1.Append(chars[c]);

                if (Hankaku.IsHankaku(chars[c]))
                {
                    length++;
                }
                else
                {
                    length += 2;
                }

                c++;
            }

            // column2より前
            while (c < chars.Length && length < column2)
            {
                sb2.Append(chars[c]);

                if (Hankaku.IsHankaku(chars[c]))
                {
                    length++;
                }
                else
                {
                    length += 2;
                }

                c++;
            }

            // 残り
            while ( c < chars.Length )
            {
                sb3.Append(chars[c]);
                c++;
            }

            result = new string[]{sb1.ToString(),sb2.ToString(),sb3.ToString()};
        }


        public static bool IsHankaku(char ch)
        {
            bool matched = false;

            if (
                Hankaku.IsHankakuSymbol(ch)//半角記号
                || Hankaku.IsHankakuSuji(ch)//半角数字
                || Hankaku.IsHankakuOhmoji(ch)//半角英字大文字
                || Hankaku.IsHankakuKomoji(ch)//半角英字小文字
                || Hankaku.IsHankakuKana(ch)//半角カナ
                )
            {
                matched = true;
            }

            return matched;
        }

        /// <summary>
        /// 半角大文字。
        /// </summary>
        /// <param name="ch"></param>
        /// <returns></returns>
        public static bool IsHankakuOhmoji(char ch)
        {
            return 'A' <= ch && ch <= 'Z';
        }

        /// <summary>
        /// 半角小文字。
        /// </summary>
        /// <param name="ch"></param>
        /// <returns></returns>
        public static bool IsHankakuKomoji(char ch)
        {
            return 'a' <= ch && ch <= 'z';
        }

        /// <summary>
        /// 半角カナ。
        /// </summary>
        /// <param name="ch"></param>
        /// <returns></returns>
        public static bool IsHankakuKana(char ch)
        {
            return
                ch == '｡'
                || ch == '｢'
                || ch == '｣'
                || ch == '、'
                || ch == '･'
                || ch == 'ﾞ'
                || ch == 'ﾟ'
                || ch == 'ｦ'
                || 'ｧ' <= ch && ch <= 'ｯ'
                || ch == 'ｰ'
                || 'ｱ' <= ch && ch <= 'ﾝ'
                ;
        }

        /// <summary>
        /// 半角記号。
        /// </summary>
        /// <param name="ch"></param>
        public static bool IsHankakuSymbol(char ch)
        {
            return
                ch==' '
                || ch == '!'
                || ch == '"'
                || ch == '#'
                || ch == '$'
                || ch == '%'
                || ch == '&'
                || ch == '\''
                || ch == '('
                || ch == ')'
                || ch == '*'
                || ch == '+'
                || ch == ','
                || ch == '-'
                || ch == '.'

                || ch == ':'
                || ch == ';'
                || ch == '<'
                || ch == '='
                || ch == '>'
                || ch == '?'
                || ch == '@'

                || ch == '/'
                || ch == '['
                || ch == '\\'
                || ch == ']'
                || ch == '^'
                || ch == '_'
                || ch == '`'
                || ch == '{'
                || ch == '|'
                || ch == '}'
                || ch == '~'
                ;
        }


        /// <summary>
        /// 半角数字。
        /// </summary>
        /// <param name="ch"></param>
        public static bool IsHankakuSuji(char ch)
        {
            return
                ch == '0'
                || ch == '1'
                || ch == '2'
                || ch == '3'
                || ch == '4'
                || ch == '5'
                || ch == '6'
                || ch == '7'
                || ch == '8'
                || ch == '9'
                ;
        }

    }
}
